@@ -1,23 +1,5 @@
 package SQL::Translator::Filter::Names;
 
-# -------------------------------------------------------------------
-# Copyright (C) 2002-2009 SQLFairy Authors
-#
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License as
-# published by the Free Software Foundation; version 2.
-#
-# This program is distributed in the hope that it will be useful, but
-# WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-# 02111-1307  USA
-# -------------------------------------------------------------------
-
 =head1 NAME
 
 SQL::Translator::Filter::Names - Tweak the names of schema objects.
@@ -46,8 +28,8 @@ SQL::Translator::Filter::Names - Tweak the names of schema objects.
 =cut
 
 use strict;
-use vars qw/$VERSION/;
-$VERSION = '1.59';
+use warnings;
+our $VERSION = '1.59';
 
 sub filter {
     my $schema = shift;
