@@ -136,7 +136,7 @@ sub enum_constraint {
   return (
      'CONSTRAINT ' . $self->enum_constraint_name($field_name) .
        ' CHECK (' . $self->quote($field_name) .
-       ' IN (' . join( ',', map $self->quote_string($_), @$vals ) . '))'
+       ' IN (' . join( ',', map qq('$_'), @$vals ) . '))'
   )
 }
 
@@ -198,7 +198,7 @@ sub remove_table_constraints {
 }
 
 sub drop_tables {
-   my ($self, $schema) = @_;
+   my ($self, $schema) = shift;
 
    if ($self->add_drop_table) {
       my @tables = sort { $b->order <=> $a->order } $schema->get_tables;
