@@ -78,13 +78,12 @@ sub _ipk {
 sub field {
    my ($self, $field) = @_;
 
+
    return join ' ',
       $self->field_comments($field),
       $self->field_name($field),
       ( $self->_ipk($field)
-         ? $field->is_auto_increment
-            ? ( 'INTEGER PRIMARY KEY AUTOINCREMENT' )
-            : ( 'INTEGER PRIMARY KEY' )
+         ? ( 'INTEGER PRIMARY KEY' )
          : ( $self->field_type($field) )
       ),
       $self->field_nullable($field),
