@@ -1,23 +1,5 @@
 package SQL::Translator::Producer::PostgreSQL;
 
-# -------------------------------------------------------------------
-# Copyright (C) 2002-2009 SQLFairy Authors
-#
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License as
-# published by the Free Software Foundation; version 2.
-#
-# This program is distributed in the hope that it will be useful, but
-# WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-# 02111-1307  USA
-# -------------------------------------------------------------------
-
 =head1 NAME
 
 SQL::Translator::Producer::PostgreSQL - PostgreSQL producer for SQL::Translator
@@ -34,20 +16,23 @@ producer.
 
 Now handles PostGIS Geometry and Geography data types on table definitions.
 Does not yet support PostGIS Views.
-	
+
 =cut
 
 use strict;
 use warnings;
-use vars qw[ $DEBUG $WARN $VERSION %used_names ];
-$VERSION = '1.59';
+our ( $DEBUG, $WARN );
+our $VERSION = '1.59';
 $DEBUG = 0 unless defined $DEBUG;
 
 use base qw(SQL::Translator::Producer);
 use SQL::Translator::Schema::Constants;
 use SQL::Translator::Utils qw(debug header_comment parse_dbms_version);
+use SQL::Translator::Generator::DDL::PostgreSQL;
 use Data::Dumper;
 
+my $generator = SQL::Translator::Generator::DDL::PostgreSQL->new;
+
 my ( %translate, %index_name );
 my $max_id_length;
 
@@ -111,20 +96,20 @@ BEGIN {
  $max_id_length = 62;
 }
 my %reserved = map { $_, 1 } qw[
-    ALL ANALYSE ANALYZE AND ANY AS ASC 
+    ALL ANALYSE ANALYZE AND ANY AS ASC
     BETWEEN BINARY BOTH
     CASE CAST CHECK COLLATE COLUMN CONSTRAINT CROSS
-    CURRENT_DATE CURRENT_TIME CURRENT_TIMESTAMP CURRENT_USER 
+    CURRENT_DATE CURRENT_TIME CURRENT_TIMESTAMP CURRENT_USER
     DEFAULT DEFERRABLE DESC DISTINCT DO
     ELSE END EXCEPT
-    FALSE FOR FOREIGN FREEZE FROM FULL 
-    GROUP HAVING 
-    ILIKE IN INITIALLY INNER INTERSECT INTO IS ISNULL 
-    JOIN LEADING LEFT LIKE LIMIT 
+    FALSE FOR FOREIGN FREEZE FROM FULL
+    GROUP HAVING
+    ILIKE IN INITIALLY INNER INTERSECT INTO IS ISNULL
+    JOIN LEADING LEFT LIKE LIMIT
     NATURAL NEW NOT NOTNULL NULL
     OFF OFFSET OLD ON ONLY OR ORDER OUTER OVERLAPS
-    PRIMARY PUBLIC REFERENCES RIGHT 
-    SELECT SESSION_USER SOME TABLE THEN TO TRAILING TRUE 
+    PRIMARY PUBLIC REFERENCES RIGHT
+    SELECT SESSION_USER SOME TABLE THEN TO TRAILING TRUE
     UNION UNIQUE USER USING VERBOSE WHEN WHERE
 ];
 
@@ -174,7 +159,6 @@ and table_constraint is:
 
 =cut
 
-# -------------------------------------------------------------------
 sub produce {
     my $translator       = shift;
     local $DEBUG         = $translator->debug;
@@ -189,7 +173,8 @@ sub produce {
 
     my $qt = $translator->quote_table_names ? q{"} : q{};
     my $qf = $translator->quote_field_names ? q{"} : q{};
-    
+    $generator->quote_chars([$qt]);
+
     my @output;
     push @output, header_comment unless ($no_comments);
 
@@ -197,7 +182,7 @@ sub produce {
     my %type_defs;
     for my $table ( $schema->get_tables ) {
 
-        my ($table_def, $fks) = create_table($table, { 
+        my ($table_def, $fks) = create_table($table, {
             quote_table_names => $qt,
             quote_field_names => $qf,
             no_comments       => $no_comments,
@@ -220,6 +205,13 @@ sub produce {
       });
     }
 
+    for my $trigger ( $schema->get_triggers ) {
+      push @table_defs, create_trigger( $trigger, {
+          add_drop_trigger => $add_drop_table,
+          no_comments      => $no_comments,
+        });
+    }
+
     push @output, map { "$_;\n\n" } values %type_defs;
     push @output, map { "$_;\n\n" } @table_defs;
     if ( @fks ) {
@@ -239,18 +231,17 @@ sub produce {
         : join ('', @output);
 }
 
-# -------------------------------------------------------------------
 sub mk_name {
-    my $basename      = shift || ''; 
-    my $type          = shift || ''; 
-    my $scope         = shift || ''; 
+    my $basename      = shift || '';
+    my $type          = shift || '';
+    my $scope         = shift || '';
     my $critical      = shift || '';
     my $basename_orig = $basename;
 #    my $max_id_length = 62;
-    my $max_name      = $type 
-                        ? $max_id_length - (length($type) + 1) 
+    my $max_name      = $type
+                        ? $max_id_length - (length($type) + 1)
                         : $max_id_length;
-    $basename         = substr( $basename, 0, $max_name ) 
+    $basename         = substr( $basename, 0, $max_name )
                         if length( $basename ) > $max_name;
     my $name          = $type ? "${type}_$basename" : $basename;
 
@@ -265,7 +256,7 @@ sub mk_name {
     if ( my $prev = $scope->{ $name } ) {
         my $name_orig = $name;
         $name        .= sprintf( "%02d", ++$prev );
-        substr($name, $max_id_length - 3) = "00" 
+        substr($name, $max_id_length - 3) = "00"
             if length( $name ) > $max_id_length;
 
         warn "The name '$name_orig' has been changed to ",
@@ -278,28 +269,10 @@ sub mk_name {
     return $name;
 }
 
-# -------------------------------------------------------------------
-sub next_unused_name {
-    my $orig_name = shift or return;
-    my $name      = $orig_name;
-
-    my $suffix_gen = sub {
-        my $suffix = 0;
-        return ++$suffix ? '' : $suffix;
-    };
-
-    for (;;) {
-        $name = $orig_name . $suffix_gen->();
-        last if $used_names{ $name }++;
-    }
-
-    return $name;
-}
-
 sub is_geometry
 {
-	my $field = shift;
-	return 1 if $field->data_type eq 'geometry';
+   my $field = shift;
+   return 1 if $field->data_type eq 'geometry';
 }
 
 sub is_geography
@@ -308,12 +281,13 @@ sub is_geography
     return 1 if $field->data_type eq 'geography';
 }
 
-sub create_table 
+sub create_table
 {
     my ($table, $options) = @_;
 
     my $qt = $options->{quote_table_names} || '';
     my $qf = $options->{quote_field_names} || '';
+    $generator->quote_chars([$qt]);
     my $no_comments = $options->{no_comments} || 0;
     my $add_drop_table = $options->{add_drop_table} || 0;
     my $postgres_version = $options->{postgres_version} || 0;
@@ -321,10 +295,9 @@ sub create_table
 
     my $table_name = $table->name or next;
     my ( $fql_tbl_name ) = ( $table_name =~ s/\W(.*)$// ) ? $1 : q{};
-    my $table_name_ur = $qt ? $table_name
+    my $table_name_ur = $qt ? join('.', $table_name, $fql_tbl_name)
         : $fql_tbl_name ? join('.', $table_name, $fql_tbl_name)
         : $table_name;
-    $table->name($table_name_ur);
 
 # print STDERR "$table_name table_name\n";
     my ( @comments, @field_defs, @sequence_defs, @constraint_defs, @fks );
@@ -358,7 +331,7 @@ sub create_table
  #   my $idx_name_default;
     for my $index ( $table->get_indices ) {
         my ($idef, $constraints) = create_index($index,
-                                              { 
+                                              {
                                                   quote_field_names => $qf,
                                                   quote_table_names => $qt,
                                                   table_name => $table_name,
@@ -372,8 +345,8 @@ sub create_table
     #
     my $c_name_default;
     for my $c ( $table->get_constraints ) {
-        my ($cdefs, $fks) = create_constraint($c, 
-                                              { 
+        my ($cdefs, $fks) = create_constraint($c,
+                                              {
                                                   quote_field_names => $qf,
                                                   quote_table_names => $qt,
                                                   table_name => $table_name,
@@ -385,20 +358,20 @@ sub create_table
 
     my $temporary = "";
 
-    if(exists $table->{extra}{temporary}) {
-        $temporary = $table->{extra}{temporary} ? "TEMPORARY " : "";
-    } 
+    if(exists $table->extra->{temporary}) {
+        $temporary = $table->extra->{temporary} ? "TEMPORARY " : "";
+    }
 
     my $create_statement;
     $create_statement = join("\n", @comments);
     if ($add_drop_table) {
         if ($postgres_version >= 8.002) {
-            $create_statement .= qq[DROP TABLE IF EXISTS $qt$table_name_ur$qt CASCADE;\n];
+            $create_statement .= 'DROP TABLE IF EXISTS ' . $generator->quote($table_name_ur) . " CASCADE;\n";
         } else {
-            $create_statement .= qq[DROP TABLE $qt$table_name_ur$qt CASCADE;\n];
+            $create_statement .= 'DROP TABLE ' . $generator->quote($table_name_ur) . " CASCADE;\n";
         }
     }
-    $create_statement .= qq[CREATE ${temporary}TABLE $qt$table_name_ur$qt (\n].
+    $create_statement .= "CREATE ${temporary}TABLE " . $generator->quote($table_name_ur) . " (\n" .
                             join( ",\n", map { "  $_" } @field_defs, @constraint_defs ).
                             "\n)"
                             ;
@@ -406,16 +379,16 @@ sub create_table
     $create_statement .= ( $create_statement =~ /;$/ ? "\n" : q{} )
         . join(";\n", @index_defs);
 
-	#
-	# Geometry
-	#
-	if(grep { is_geometry($_) } $table->get_fields){
+   #
+   # Geometry
+   #
+   if(grep { is_geometry($_) } $table->get_fields){
         $create_statement .= ";";
         my @geometry_columns;
         foreach my $col ($table->get_fields) { push(@geometry_columns,$col) if is_geometry($col); }
-		$create_statement .= "\n".join("\n", map{ drop_geometry_column($_) } @geometry_columns) if $options->{add_drop_table};
-		$create_statement .= "\n".join("\n", map{ add_geometry_column($_) } @geometry_columns);
-	}
+      $create_statement .= "\n".join("\n", map{ drop_geometry_column($_) } @geometry_columns) if $options->{add_drop_table};
+      $create_statement .= "\n".join("\n", map{ add_geometry_column($_) } @geometry_columns);
+   }
 
     return $create_statement, \@fks;
 }
@@ -424,6 +397,7 @@ sub create_view {
     my ($view, $options) = @_;
     my $qt = $options->{quote_table_names} || '';
     my $qf = $options->{quote_field_names} || '';
+    $generator->quote_chars([$qt]);
     my $postgres_version = $options->{postgres_version} || 0;
     my $add_drop_view = $options->{add_drop_view};
 
@@ -431,23 +405,23 @@ sub create_view {
     debug("PKG: Looking at view '${view_name}'\n");
 
     my $create = '';
-    $create .= "--\n-- View: ${qt}${view_name}${qt}\n--\n"
+    $create .= "--\n-- View: " . $generator->quote($view_name) . "\n--\n"
         unless $options->{no_comments};
     if ($add_drop_view) {
         if ($postgres_version >= 8.002) {
-            $create .= "DROP VIEW IF EXISTS ${qt}${view_name}${qt};\n";
+            $create .= "DROP VIEW IF EXISTS " . $generator->quote($view_name) . ";\n";
         } else {
-            $create .= "DROP VIEW ${qt}${view_name}${qt};\n";
+            $create .= "DROP VIEW " . $generator->quote($view_name) . ";\n";
         }
     }
     $create .= 'CREATE';
 
     my $extra = $view->extra;
     $create .= " TEMPORARY" if exists($extra->{temporary}) && $extra->{temporary};
-    $create .= " VIEW ${qt}${view_name}${qt}";
+    $create .= " VIEW " . $generator->quote($view_name);
 
     if ( my @fields = $view->fields ) {
-        my $field_list = join ', ', map { "${qf}${_}${qf}" } @fields;
+        my $field_list = join ', ', map { $generator->quote($_) } @fields;
         $create .= " ( ${field_list} )";
     }
 
@@ -462,7 +436,7 @@ sub create_view {
     return $create;
 }
 
-{ 
+{
 
     my %field_name_scope;
 
@@ -472,6 +446,7 @@ sub create_view {
 
         my $qt = $options->{quote_table_names} || '';
         my $qf = $options->{quote_field_names} || '';
+        $generator->quote_chars([$qt]);
         my $table_name = $field->table->name;
         my $constraint_defs = $options->{constraint_defs} || [];
         my $postgres_version = $options->{postgres_version} || 0;
@@ -479,11 +454,11 @@ sub create_view {
 
         $field_name_scope{$table_name} ||= {};
         my $field_name    = $field->name;
-        my $field_comments = $field->comments 
-            ? "-- " . $field->comments . "\n  " 
+        my $field_comments = $field->comments
+            ? "-- " . $field->comments . "\n  "
             : '';
 
-        my $field_def     = $field_comments.qq[$qf$field_name$qf];
+        my $field_def     = $field_comments . $generator->quote($field_name);
 
         #
         # Datatype
@@ -510,7 +485,7 @@ sub create_view {
         }
 
         #
-        # Default value 
+        # Default value
         #
         SQL::Translator::Producer->_apply_default_value(
           $field,
@@ -527,51 +502,51 @@ sub create_view {
         #
         $field_def .= ' NOT NULL' unless $field->is_nullable;
 
-		#
-		# Geometry constraints
-		#
-		if(is_geometry($field)){
-			foreach ( create_geometry_constraints($field) ) {
-				my ($cdefs, $fks) = create_constraint($_, 
-													  { 
-														  quote_field_names => $qf,
-														  quote_table_names => $qt,
-														  table_name => $table_name,
-													  });
-				push @$constraint_defs, @$cdefs;
-				push @$fks, @$fks;
-			}
+      #
+      # Geometry constraints
+      #
+      if(is_geometry($field)){
+         foreach ( create_geometry_constraints($field) ) {
+            my ($cdefs, $fks) = create_constraint($_,
+                                         {
+                                            quote_field_names => $qf,
+                                            quote_table_names => $qt,
+                                            table_name => $table_name,
+                                         });
+            push @$constraint_defs, @$cdefs;
+            push @$fks, @$fks;
+         }
         }
-		
+
         return $field_def;
     }
 }
 
 sub create_geometry_constraints{
-	my $field = shift;
+   my $field = shift;
 
-	my @constraints;
-	push @constraints, SQL::Translator::Schema::Constraint->new(
-							name       => "enforce_dims_".$field->name,
-							expression => "(ST_NDims($field) = ".$field->{extra}{dimensions}.")",
-							table 	   => $field->table,
-							type       => CHECK_C,
-						);
-						
-	push @constraints, SQL::Translator::Schema::Constraint->new(
-							name       => "enforce_srid_".$field->name,
-							expression => "(ST_SRID($field) = ".$field->{extra}{srid}.")",
-							table 	   => $field->table,
-							type       => CHECK_C,
-						);
-	push @constraints, SQL::Translator::Schema::Constraint->new(
-							name       => "enforce_geotype_".$field->name,
-							expression => "(GeometryType($field) = '".$field->{extra}{geometry_type}."'::text OR $field IS NULL)",
-							table 	   => $field->table,
-							type       => CHECK_C,
-						);
-						
-	return @constraints;
+   my @constraints;
+   push @constraints, SQL::Translator::Schema::Constraint->new(
+                     name       => "enforce_dims_".$field->name,
+                     expression => "(ST_NDims($field) = ".$field->extra->{dimensions}.")",
+                     table       => $field->table,
+                     type       => CHECK_C,
+                  );
+
+   push @constraints, SQL::Translator::Schema::Constraint->new(
+                     name       => "enforce_srid_".$field->name,
+                     expression => "(ST_SRID($field) = ".$field->extra->{srid}.")",
+                     table       => $field->table,
+                     type       => CHECK_C,
+                  );
+   push @constraints, SQL::Translator::Schema::Constraint->new(
+                     name       => "enforce_geotype_".$field->name,
+                     expression => "(GeometryType($field) = '".$field->extra->{geometry_type}."'::text OR $field IS NULL)",
+                     table       => $field->table,
+                     type       => CHECK_C,
+                  );
+
+   return @constraints;
 }
 
 sub create_index
@@ -580,21 +555,21 @@ sub create_index
 
     my $qt = $options->{quote_table_names} ||'';
     my $qf = $options->{quote_field_names} ||'';
+    $generator->quote_chars([$qt]);
     my $table_name = $index->table->name;
 
     my ($index_def, @constraint_defs);
 
-    my $name = next_unused_name(
-        $index->name 
-        || join('_', $table_name, 'idx', ++$index_name{ $table_name })
-    );
+    my $name
+        = $index->name
+        || join('_', $table_name, 'idx', ++$index_name{ $table_name });
 
     my $type = $index->type || NORMAL;
     my @fields     =  $index->fields;
-    next unless @fields;
+    return unless @fields;
 
-    my $def_start = qq[CONSTRAINT ${qf}$name${qf} ];
-    my $field_names = '(' . join(", ", (map { $_ =~ /\(.*\)/ ? $_ : ($qf . $_ . $qf ) } @fields)) . ')';
+    my $def_start = 'CONSTRAINT ' . $generator->quote($name) . ' ';
+    my $field_names = '(' . join(", ", (map { $_ =~ /\(.*\)/ ? $_ : ( $generator->quote($_) ) } @fields)) . ')';
     if ( $type eq PRIMARY_KEY ) {
         push @constraint_defs, "${def_start}PRIMARY KEY ".$field_names;
     }
@@ -602,9 +577,9 @@ sub create_index
         push @constraint_defs, "${def_start}UNIQUE " .$field_names;
     }
     elsif ( $type eq NORMAL ) {
-        $index_def = 
-            "CREATE INDEX ${qf}${name}${qf} on ${qt}${table_name}${qt} ".$field_names
-            ; 
+        $index_def =
+            'CREATE INDEX ' . $generator->quote($name) . ' on ' . $generator->quote($table_name) . ' ' . $field_names
+            ;
     }
     else {
         warn "Unknown index type ($type) on table $table_name.\n"
@@ -620,21 +595,19 @@ sub create_constraint
 
     my $qf = $options->{quote_field_names} ||'';
     my $qt = $options->{quote_table_names} ||'';
+    $generator->quote_chars([$qt]);
     my $table_name = $c->table->name;
     my (@constraint_defs, @fks);
 
     my $name = $c->name || '';
-    if ( $name ) {
-        $name = next_unused_name($name);
-    }
 
     my @fields = grep { defined } $c->fields;
 
     my @rfields = grep { defined } $c->reference_fields;
 
     next if !@fields && $c->type ne CHECK_C;
-    my $def_start = $name ? qq[CONSTRAINT ${qf}$name${qf} ] : '';
-    my $field_names = '(' . join(", ", (map { $_ =~ /\(.*\)/ ? $_ : ($qf . $_ . $qf ) } @fields)) . ')';
+    my $def_start = $name ? 'CONSTRAINT ' . $generator->quote($name) . ' ' : '';
+    my $field_names = '(' . join(", ", (map { $_ =~ /\(.*\)/ ? $_ : ( $generator->quote($_) ) } @fields)) . ')';
     if ( $c->type eq PRIMARY_KEY ) {
         push @constraint_defs, "${def_start}PRIMARY KEY ".$field_names;
     }
@@ -646,24 +619,24 @@ sub create_constraint
         push @constraint_defs, "${def_start}CHECK ($expression)";
     }
     elsif ( $c->type eq FOREIGN_KEY ) {
-        my $def .= "ALTER TABLE ${qt}${table_name}${qt} ADD FOREIGN KEY " . $field_names .
-            "\n  REFERENCES " . $qt . $c->reference_table . $qt;
+        my $def .= "ALTER TABLE " . $generator->quote($table_name) . " ADD ${def_start}FOREIGN KEY $field_names"
+            . "\n  REFERENCES " . $generator->quote($c->reference_table);
 
         if ( @rfields ) {
-            $def .= ' ('.$qf . join( $qf.', '.$qf, @rfields ) . $qf.')';
+            $def .= ' (' . join( ', ', map { $generator->quote($_) } @rfields ) . ')';
         }
 
         if ( $c->match_type ) {
-            $def .= ' MATCH ' . 
+            $def .= ' MATCH ' .
                 ( $c->match_type =~ /full/i ) ? 'FULL' : 'PARTIAL';
         }
 
         if ( $c->on_delete ) {
-            $def .= ' ON DELETE '.join( ' ', $c->on_delete );
+            $def .= ' ON DELETE '. $c->on_delete;
         }
 
         if ( $c->on_update ) {
-            $def .= ' ON UPDATE '.join( ' ', $c->on_update );
+            $def .= ' ON UPDATE '. $c->on_update;
         }
 
         if ( $c->deferrable ) {
@@ -676,6 +649,30 @@ sub create_constraint
     return \@constraint_defs, \@fks;
 }
 
+sub create_trigger {
+  my ($trigger,$options) = @_;
+
+  my @statements;
+
+  push @statements, sprintf( 'DROP TRIGGER IF EXISTS %s', $trigger->name )
+    if $options->{add_drop_trigger};
+
+  my $scope = $trigger->scope || '';
+  $scope = " FOR EACH $scope" if $scope;
+
+  push @statements, sprintf(
+    'CREATE TRIGGER %s %s %s ON %s%s %s',
+    $trigger->name,
+    $trigger->perform_action_when,
+    join( ' OR ', @{ $trigger->database_events } ),
+    $trigger->on_table,
+    $scope,
+    $trigger->action,
+  );
+
+  return @statements;
+}
+
 sub convert_datatype
 {
     my ($field) = @_;
@@ -688,8 +685,8 @@ sub convert_datatype
 #        my $len = 0;
 #        $len = ($len < length($_)) ? length($_) : $len for (@$list);
 #        my $chk_name = mk_name( $table_name.'_'.$field_name, 'chk' );
-#        push @$constraint_defs, 
-#        qq[CONSTRAINT "$chk_name" CHECK ($qf$field_name$qf ].
+#        push @$constraint_defs,
+#        'CONSTRAINT "$chk_name" CHECK (' . $generator->quote(field_name) .
 #           qq[IN ($commalist))];
         $data_type = 'character varying';
     }
@@ -740,7 +737,7 @@ sub convert_datatype
     );
 
     if ( $data_type !~ /$type_with_size/ ) {
-        @size = (); 
+        @size = ();
     }
 
     if (defined $size[0] && $size[0] > 0 && $data_type =~ /^time/i ) {
@@ -758,7 +755,7 @@ sub convert_datatype
     # Geography
     #
     if($data_type eq 'geography'){
-        $data_type .= '('.$field->{extra}{geography_type}.','. $field->{extra}{srid} .')'
+        $data_type .= '('.$field->extra->{geography_type}.','. $field->extra->{srid} .')'
     }
 
     return $data_type;
@@ -769,15 +766,25 @@ sub alter_field
 {
     my ($from_field, $to_field) = @_;
 
-    die "Can't alter field in another table" 
+    die "Can't alter field in another table"
         if($from_field->table->name ne $to_field->table->name);
 
     my @out;
-    
+
     # drop geometry column and constraints
-	push @out, drop_geometry_column($from_field) if is_geometry($from_field);
-	push @out, drop_geometry_constraints($from_field) if is_geometry($from_field);
-    
+    push @out, drop_geometry_column($from_field) if is_geometry($from_field);
+    push @out, drop_geometry_constraints($from_field) if is_geometry($from_field);
+
+    # it's necessary to start with rename column cause this would affect
+    # all of the following statements which would be broken if do the
+    # rename later
+    # BUT: drop geometry is done before the rename, cause it work's on the
+    # $from_field directly
+    push @out, sprintf('ALTER TABLE %s RENAME COLUMN %s TO %s',
+                       $to_field->table->name,
+                       $from_field->name,
+                       $to_field->name) if($from_field->name ne $to_field->name);
+
     push @out, sprintf('ALTER TABLE %s ALTER COLUMN %s SET NOT NULL',
                        $to_field->table->name,
                        $to_field->name) if(!$to_field->is_nullable and
@@ -796,15 +803,10 @@ sub alter_field
                        $to_field->name,
                        $to_dt) if($to_dt ne $from_dt);
 
-    push @out, sprintf('ALTER TABLE %s RENAME COLUMN %s TO %s',
-                       $to_field->table->name,
-                       $from_field->name,
-                       $to_field->name) if($from_field->name ne $to_field->name);
-
     my $old_default = $from_field->default_value;
     my $new_default = $to_field->default_value;
     my $default_value = $to_field->default_value;
-    
+
     # fixes bug where output like this was created:
     # ALTER TABLE users ALTER COLUMN column SET DEFAULT ThisIsUnescaped;
     if(ref $default_value eq "SCALAR" ) {
@@ -813,7 +815,7 @@ sub alter_field
         $default_value =~ s/'/''/xsmg;
         $default_value = q(') . $default_value . q(');
     }
-    
+
     push @out, sprintf('ALTER TABLE %s ALTER COLUMN %s SET DEFAULT %s',
                        $to_field->table->name,
                        $to_field->name,
@@ -821,19 +823,19 @@ sub alter_field
         if ( defined $new_default &&
              (!defined $old_default || $old_default ne $new_default) );
 
-     # fixes bug where removing the DEFAULT statement of a column
-     # would result in no change
-    
-     push @out, sprintf('ALTER TABLE %s ALTER COLUMN %s DROP DEFAULT',
+    # fixes bug where removing the DEFAULT statement of a column
+    # would result in no change
+
+    push @out, sprintf('ALTER TABLE %s ALTER COLUMN %s DROP DEFAULT',
                        $to_field->table->name,
                        $to_field->name)
         if ( !defined $new_default && defined $old_default );
-    
-	# add geometry column and contraints
-	push @out, add_geometry_column($to_field) if is_geometry($to_field);
-	push @out, add_geometry_constraints($to_field) if is_geometry($to_field);
-	
-    return wantarray ? @out : join("\n", @out);
+
+    # add geometry column and constraints
+    push @out, add_geometry_column($to_field) if is_geometry($to_field);
+    push @out, add_geometry_constraints($to_field) if is_geometry($to_field);
+
+    return wantarray ? @out : join(";\n", @out);
 }
 
 sub rename_field { alter_field(@_) }
@@ -853,65 +855,70 @@ sub add_field
 
 sub drop_field
 {
-    my ($old_field) = @_;
+    my ($old_field, $options) = @_;
+
+    my $qt = $options->{quote_table_names} ||'';
+    my $qf = $options->{quote_field_names} ||'';
+    $generator->quote_chars([$qt]);
 
     my $out = sprintf('ALTER TABLE %s DROP COLUMN %s',
-                      $old_field->table->name,
-                      $old_field->name);
-	$out .= "\n".drop_geometry_column($old_field) if is_geometry($old_field);
-    return $out;    
+                      $generator->quote($old_field->table->name),
+                      $generator->quote($old_field->name));
+        $out .= "\n".drop_geometry_column($old_field) if is_geometry($old_field);
+    return $out;
 }
 
 sub add_geometry_column{
-	my ($field,$options) = @_;
-	
-	my $out = sprintf("INSERT INTO geometry_columns VALUES ('%s','%s','%s','%s','%s','%s','%s')",
-						'',
-						$field->table->schema->name,
-						$options->{table} ? $options->{table} : $field->table->name,
-						$field->name,
-						$field->{extra}{dimensions},
-						$field->{extra}{srid},
-						$field->{extra}{geometry_type});
+   my ($field,$options) = @_;
+
+   my $out = sprintf("INSERT INTO geometry_columns VALUES ('%s','%s','%s','%s','%s','%s','%s')",
+                  '',
+                  $field->table->schema->name,
+                  $options->{table} ? $options->{table} : $field->table->name,
+                  $field->name,
+                  $field->extra->{dimensions},
+                  $field->extra->{srid},
+                  $field->extra->{geometry_type});
     return $out;
 }
 
 sub drop_geometry_column
 {
-	my $field = shift;
-	
-	my $out = sprintf("DELETE FROM geometry_columns WHERE f_table_schema = '%s' AND f_table_name = '%s' AND f_geometry_column = '%s'",
-						$field->table->schema->name,
-						$field->table->name,
-						$field->name);
+   my $field = shift;
+
+   my $out = sprintf("DELETE FROM geometry_columns WHERE f_table_schema = '%s' AND f_table_name = '%s' AND f_geometry_column = '%s'",
+                  $field->table->schema->name,
+                  $field->table->name,
+                  $field->name);
     return $out;
 }
 
 sub add_geometry_constraints{
-	my $field = shift;
-	
-	my @constraints = create_geometry_constraints($field);
+   my $field = shift;
 
-	my $out = join("\n", map { alter_create_constraint($_); } @constraints);
-	
-	return $out;
+   my @constraints = create_geometry_constraints($field);
+
+   my $out = join("\n", map { alter_create_constraint($_); } @constraints);
+
+   return $out;
 }
 
 sub drop_geometry_constraints{
-	my $field = shift;
-	
-	my @constraints = create_geometry_constraints($field);
-	
-	my $out = join("\n", map { alter_drop_constraint($_); } @constraints);
-	
-	return $out;
+   my $field = shift;
+
+   my @constraints = create_geometry_constraints($field);
+
+   my $out = join("\n", map { alter_drop_constraint($_); } @constraints);
+
+   return $out;
 }
 
 sub alter_table {
     my ($to_table, $options) = @_;
     my $qt = $options->{quote_table_names} || '';
+    $generator->quote_chars([$qt]);
     my $out = sprintf('ALTER TABLE %s %s',
-                      $qt . $to_table->name . $qt,
+                      $generator->quote($to_table->name),
                       $options->{alter_table_action});
     $out .= "\n".$options->{geometry_changes} if $options->{geometry_changes};
     return $out;
@@ -920,14 +927,15 @@ sub alter_table {
 sub rename_table {
     my ($old_table, $new_table, $options) = @_;
     my $qt = $options->{quote_table_names} || '';
-    $options->{alter_table_action} = "RENAME TO $qt$new_table$qt";
+    $generator->quote_chars([$qt]);
+    $options->{alter_table_action} = "RENAME TO " . $generator->quote($new_table);
+
+   my @geometry_changes;
+   push @geometry_changes, map { drop_geometry_column($_); } grep { is_geometry($_) } $old_table->get_fields;
+   push @geometry_changes, map { add_geometry_column($_, { table => $new_table }); } grep { is_geometry($_) } $old_table->get_fields;
 
-	my @geometry_changes;
-	push @geometry_changes, map { drop_geometry_column($_); } grep { is_geometry($_) } $old_table->get_fields;
-	push @geometry_changes, map { add_geometry_column($_, { table => $new_table }); } grep { is_geometry($_) } $old_table->get_fields;
-	
     $options->{geometry_changes} = join ("\n",@geometry_changes) if scalar(@geometry_changes);
-    
+
     return alter_table($old_table, $options);
 }
 
@@ -935,6 +943,7 @@ sub alter_create_index {
     my ($index, $options) = @_;
     my $qt = $options->{quote_table_names} || '';
     my $qf = $options->{quote_field_names} || '';
+    $generator->quote_chars([$qt]);
     my ($idef, $constraints) = create_index($index, {
         quote_field_names => $qf,
         quote_table_names => $qt,
@@ -942,7 +951,7 @@ sub alter_create_index {
     });
     return $index->type eq NORMAL ? $idef
         : sprintf('ALTER TABLE %s ADD %s',
-              $qt . $index->table->name . $qt,
+              $generator->quote($index->table->name),
               join(q{}, @$constraints)
           );
 }
@@ -957,24 +966,43 @@ sub alter_drop_constraint {
     my ($c, $options) = @_;
     my $qt = $options->{quote_table_names} || '';
     my $qc = $options->{quote_field_names} || '';
-    my $out = sprintf('ALTER TABLE %s DROP CONSTRAINT %s',
-                      $qt . $c->table->name . $qt,
-                      $qc . $c->name . $qc );
-    return $out;
+    $generator->quote_chars([$qt]);
+
+    # attention: Postgres  has a very special naming structure for naming
+    # foreign keys and primary keys.  It names them using the name of the
+    # table as prefix and fkey or pkey as suffix, concatenated by an underscore
+    my $c_name;
+    if( $c->name ) {
+        # Already has a name, just quote it
+        $c_name = $qc . $c->name . $qc;
+    } elsif ( $c->type eq FOREIGN_KEY ) {
+        # Doesn't have a name, and is foreign key, append '_fkey'
+        $c_name = $qc . $c->table->name . '_' .
+                    ($c->fields)[0] . '_fkey' . $qc;
+    } elsif ( $c->type eq PRIMARY_KEY ) {
+        # Doesn't have a name, and is primary key, append '_pkey'
+        $c_name = $qc . $c->table->name . '_pkey' . $qc;
+    }
+
+    return sprintf(
+        'ALTER TABLE %s DROP CONSTRAINT %s',
+        $qt . $c->table->name . $qt, $c_name
+    );
 }
 
 sub alter_create_constraint {
     my ($index, $options) = @_;
     my $qt = $options->{quote_table_names} || '';
+    $generator->quote_chars([$qt]);
     my ($defs, $fks) = create_constraint(@_);
-    
+
     # return if there are no constraint definitions so we don't run
     # into output like this:
     # ALTER TABLE users ADD ;
-        
+
     return unless(@{$defs} || @{$fks});
     return $index->type eq FOREIGN_KEY ? join(q{}, @{$fks})
-        : join( ' ', 'ALTER TABLE', $qt.$index->table->name.$qt,
+        : join( ' ', 'ALTER TABLE', $generator->quote($index->table->name),
               'ADD', join(q{}, @{$defs}, @{$fks})
           );
 }
@@ -982,8 +1010,9 @@ sub alter_create_constraint {
 sub drop_table {
     my ($table, $options) = @_;
     my $qt = $options->{quote_table_names} || '';
-    my $out = "DROP TABLE $qt$table$qt CASCADE";
-    
+    $generator->quote_chars([$qt]);
+    my $out = "DROP TABLE " . $generator->quote($table) . " CASCADE";
+
     my @geometry_drops = map { drop_geometry_column($_); } grep { is_geometry($_) } $table->get_fields;
 
     $out .= "\n".join("\n",@geometry_drops) if scalar(@geometry_drops);
