@@ -26,52 +26,29 @@ returns the database structure.
 
 =cut
 
-use strict;
-use warnings;
+use Moo;
 use SQL::Translator::Schema::Constants;
 use SQL::Translator::Schema::Procedure;
 use SQL::Translator::Schema::Table;
 use SQL::Translator::Schema::Trigger;
 use SQL::Translator::Schema::View;
+use Sub::Quote qw(quote_sub);
 
 use SQL::Translator::Utils 'parse_list_arg';
 use Carp;
 
-use base 'SQL::Translator::Schema::Object';
+extends 'SQL::Translator::Schema::Object';
+
 our $VERSION = '1.59';
 
-__PACKAGE__->_attributes(qw/name database translator/);
 
-sub new {
-  my $class = shift;
-  my $self = $class->SUPER::new (@_)
-    or return;
-
-  $self->{_order} = { map { $_ => 0 } qw/
+has _order => (is => 'ro', default => quote_sub(q{ +{ map { $_ => 0 } qw/
     table
     view
     trigger
     proc
-  /};
-
-  return $self;
-}
-
-# FIXME - to be removed, together with the SQL::Translator::Schema::Graph* stuff
-# looks like a remnant of the Turnkey project integration back in 2003-4
-# Appears to be quite dead
-sub as_graph {
-
-    eval { require Class::MakeMethods }
-      or croak 'You need to install the CPAN dependency Class::MakeMethods to use as_graph()';
-
-    require  SQL::Translator::Schema::Graph;
-
-    my $self = shift;
-
-    return SQL::Translator::Schema::Graph->new(
-        translator => $self->translator );
-}
+  /} }),
+);
 
 sub as_graph_pm {
 
@@ -104,6 +81,8 @@ Returns a Graph::Directed object with the table names for nodes.
     return $g;
 }
 
+has _tables => ( is => 'ro', init_arg => undef, default => quote_sub(q{ +{} }) );
+
 sub add_table {
 
 =pod
@@ -136,16 +115,16 @@ not be created.
           or return $self->error( $table_class->error );
     }
 
-    $table->order( ++$self->{_order}{table} );
+    $table->order( ++$self->_order->{table} );
 
     # We know we have a name as the Table->new above errors if none given.
     my $table_name = $table->name;
 
-    if ( defined $self->{'tables'}{$table_name} ) {
-        return $self->error(qq[Can't create table: "$table_name" exists]);
+    if ( defined $self->_tables->{$table_name} ) {
+        return $self->error(qq[Can't use table name "$table_name": table exists]);
     }
     else {
-        $self->{'tables'}{$table_name} = $table;
+        $self->_tables->{$table_name} = $table;
     }
 
     return $table;
@@ -180,21 +159,23 @@ can be set to 1 to also drop all triggers on the table, default is 0.
     my %args    = @_;
     my $cascade = $args{'cascade'};
 
-    if ( !exists $self->{'tables'}{$table_name} ) {
-        return $self->error(qq[Can't drop table: $table_name" doesn't exist]);
+    if ( !exists $self->_tables->{$table_name} ) {
+        return $self->error(qq[Can't drop table: "$table_name" doesn't exist]);
     }
 
-    my $table = delete $self->{'tables'}{$table_name};
+    my $table = delete $self->_tables->{$table_name};
 
     if ($cascade) {
 
         # Drop all triggers on this table
         $self->drop_trigger()
-          for ( grep { $_->on_table eq $table_name } @{ $self->{'triggers'} } );
+          for ( grep { $_->on_table eq $table_name } values %{ $self->_triggers } );
     }
     return $table;
 }
 
+has _procedures => ( is => 'ro', init_arg => undef, default => quote_sub(q{ +{} }) );
+
 sub add_procedure {
 
 =pod
@@ -228,16 +209,16 @@ procedure will not be created.
           or return $self->error( $procedure_class->error );
     }
 
-    $procedure->order( ++$self->{_order}{proc} );
+    $procedure->order( ++$self->_order->{proc} );
     my $procedure_name = $procedure->name
       or return $self->error('No procedure name');
 
-    if ( defined $self->{'procedures'}{$procedure_name} ) {
+    if ( defined $self->_procedures->{$procedure_name} ) {
         return $self->error(
             qq[Can't create procedure: "$procedure_name" exists] );
     }
     else {
-        $self->{'procedures'}{$procedure_name} = $procedure;
+        $self->_procedures->{$procedure_name} = $procedure;
     }
 
     return $procedure;
@@ -269,16 +250,18 @@ object.
         $proc_name = shift;
     }
 
-    if ( !exists $self->{'procedures'}{$proc_name} ) {
+    if ( !exists $self->_procedures->{$proc_name} ) {
         return $self->error(
-            qq[Can't drop procedure: $proc_name" doesn't exist]);
+            qq[Can't drop procedure: "$proc_name" doesn't exist]);
     }
 
-    my $proc = delete $self->{'procedures'}{$proc_name};
+    my $proc = delete $self->_procedures->{$proc_name};
 
     return $proc;
 }
 
+has _triggers => ( is => 'ro', init_arg => undef, default => quote_sub(q{ +{} }) );
+
 sub add_trigger {
 
 =pod
@@ -312,14 +295,14 @@ not be created.
           or return $self->error( $trigger_class->error );
     }
 
-    $trigger->order( ++$self->{_order}{trigger} );
+    $trigger->order( ++$self->_order->{trigger} );
 
     my $trigger_name = $trigger->name or return $self->error('No trigger name');
-    if ( defined $self->{'triggers'}{$trigger_name} ) {
+    if ( defined $self->_triggers->{$trigger_name} ) {
         return $self->error(qq[Can't create trigger: "$trigger_name" exists]);
     }
     else {
-        $self->{'triggers'}{$trigger_name} = $trigger;
+        $self->_triggers->{$trigger_name} = $trigger;
     }
 
     return $trigger;
@@ -350,16 +333,18 @@ trigger name or an C<SQL::Translator::Schema::Trigger> object.
         $trigger_name = shift;
     }
 
-    if ( !exists $self->{'triggers'}{$trigger_name} ) {
+    if ( !exists $self->_triggers->{$trigger_name} ) {
         return $self->error(
-            qq[Can't drop trigger: $trigger_name" doesn't exist]);
+            qq[Can't drop trigger: "$trigger_name" doesn't exist]);
     }
 
-    my $trigger = delete $self->{'triggers'}{$trigger_name};
+    my $trigger = delete $self->_triggers->{$trigger_name};
 
     return $trigger;
 }
 
+has _views => ( is => 'ro', init_arg => undef, default => quote_sub(q{ +{} }) );
+
 sub add_view {
 
 =pod
@@ -392,14 +377,14 @@ not be created.
         $view = $view_class->new( \%args ) or return $view_class->error;
     }
 
-    $view->order( ++$self->{_order}{view} );
+    $view->order( ++$self->_order->{view} );
     my $view_name = $view->name or return $self->error('No view name');
 
-    if ( defined $self->{'views'}{$view_name} ) {
+    if ( defined $self->_views->{$view_name} ) {
         return $self->error(qq[Can't create view: "$view_name" exists]);
     }
     else {
-        $self->{'views'}{$view_name} = $view;
+        $self->_views->{$view_name} = $view;
     }
 
     return $view;
@@ -430,19 +415,15 @@ name or an C<SQL::Translator::Schema::View> object.
         $view_name = shift;
     }
 
-    if ( !exists $self->{'views'}{$view_name} ) {
-        return $self->error(qq[Can't drop view: $view_name" doesn't exist]);
+    if ( !exists $self->_views->{$view_name} ) {
+        return $self->error(qq[Can't drop view: "$view_name" doesn't exist]);
     }
 
-    my $view = delete $self->{'views'}{$view_name};
+    my $view = delete $self->_views->{$view_name};
 
     return $view;
 }
 
-sub database {
-
-=pod
-
 =head2 database
 
 Get or set the schema's database.  (optional)
@@ -451,10 +432,7 @@ Get or set the schema's database.  (optional)
 
 =cut
 
-    my $self = shift;
-    $self->{'database'} = shift if @_;
-    return $self->{'database'} || '';
-}
+has database => ( is => 'rw', default => quote_sub(q{ '' }) );
 
 sub is_valid {
 
@@ -494,8 +472,8 @@ Returns a procedure by the name provided.
     my $self = shift;
     my $procedure_name = shift or return $self->error('No procedure name');
     return $self->error(qq[Table "$procedure_name" does not exist])
-      unless exists $self->{'procedures'}{$procedure_name};
-    return $self->{'procedures'}{$procedure_name};
+      unless exists $self->_procedures->{$procedure_name};
+    return $self->_procedures->{$procedure_name};
 }
 
 sub get_procedures {
@@ -514,14 +492,14 @@ Returns all the procedures as an array or array reference.
     my @procedures =
       map  { $_->[1] }
       sort { $a->[0] <=> $b->[0] }
-      map  { [ $_->order, $_ ] } values %{ $self->{'procedures'} };
+      map  { [ $_->order, $_ ] } values %{ $self->_procedures };
 
     if (@procedures) {
         return wantarray ? @procedures : \@procedures;
     }
     else {
         $self->error('No procedures');
-        return wantarray ? () : undef;
+        return;
     }
 }
 
@@ -542,14 +520,14 @@ Returns a table by the name provided.
     my $case_insensitive = shift;
     if ( $case_insensitive ) {
       $table_name = uc($table_name);
-      foreach my $table ( keys %{$self->{tables}} ) {
-         return $self->{tables}{$table} if $table_name eq uc($table);
+      foreach my $table ( keys %{$self->_tables} ) {
+         return $self->_tables->{$table} if $table_name eq uc($table);
       }
       return $self->error(qq[Table "$table_name" does not exist]);
     }
     return $self->error(qq[Table "$table_name" does not exist])
-      unless exists $self->{'tables'}{$table_name};
-    return $self->{'tables'}{$table_name};
+      unless exists $self->_tables->{$table_name};
+    return $self->_tables->{$table_name};
 }
 
 sub get_tables {
@@ -568,14 +546,14 @@ Returns all the tables as an array or array reference.
     my @tables =
       map  { $_->[1] }
       sort { $a->[0] <=> $b->[0] }
-      map  { [ $_->order, $_ ] } values %{ $self->{'tables'} };
+      map  { [ $_->order, $_ ] } values %{ $self->_tables };
 
     if (@tables) {
         return wantarray ? @tables : \@tables;
     }
     else {
         $self->error('No tables');
-        return wantarray ? () : undef;
+        return;
     }
 }
 
@@ -593,9 +571,9 @@ Returns a trigger by the name provided.
 
     my $self = shift;
     my $trigger_name = shift or return $self->error('No trigger name');
-    return $self->error(qq[Table "$trigger_name" does not exist])
-      unless exists $self->{'triggers'}{$trigger_name};
-    return $self->{'triggers'}{$trigger_name};
+    return $self->error(qq[Trigger "$trigger_name" does not exist])
+      unless exists $self->_triggers->{$trigger_name};
+    return $self->_triggers->{$trigger_name};
 }
 
 sub get_triggers {
@@ -614,14 +592,14 @@ Returns all the triggers as an array or array reference.
     my @triggers =
       map  { $_->[1] }
       sort { $a->[0] <=> $b->[0] }
-      map  { [ $_->order, $_ ] } values %{ $self->{'triggers'} };
+      map  { [ $_->order, $_ ] } values %{ $self->_triggers };
 
     if (@triggers) {
         return wantarray ? @triggers : \@triggers;
     }
     else {
         $self->error('No triggers');
-        return wantarray ? () : undef;
+        return;
     }
 }
 
@@ -640,8 +618,8 @@ Returns a view by the name provided.
     my $self = shift;
     my $view_name = shift or return $self->error('No view name');
     return $self->error('View "$view_name" does not exist')
-      unless exists $self->{'views'}{$view_name};
-    return $self->{'views'}{$view_name};
+      unless exists $self->_views->{$view_name};
+    return $self->_views->{$view_name};
 }
 
 sub get_views {
@@ -660,14 +638,14 @@ Returns all the views as an array or array reference.
     my @views =
       map  { $_->[1] }
       sort { $a->[0] <=> $b->[0] }
-      map  { [ $_->order, $_ ] } values %{ $self->{'views'} };
+      map  { [ $_->order, $_ ] } values %{ $self->_views };
 
     if (@views) {
         return wantarray ? @views : \@views;
     }
     else {
         $self->error('No views');
-        return wantarray ? () : undef;
+        return;
     }
 }
 
@@ -677,14 +655,14 @@ sub make_natural_joins {
 
 =head2 make_natural_joins
 
-Creates foriegn key relationships among like-named fields in different
+Creates foreign key relationships among like-named fields in different
 tables.  Accepts the following arguments:
 
 =over 4
 
 =item * join_pk_only
 
-A True or False argument which determins whether or not to perform
+A True or False argument which determines whether or not to perform
 the joins from primary keys to fields of the same name in other tables
 
 =item * skip_fields
@@ -742,10 +720,6 @@ A list of fields to skip in the joins
     return 1;
 }
 
-sub name {
-
-=pod
-
 =head2 name
 
 Get or set the schema's name.  (optional)
@@ -754,12 +728,7 @@ Get or set the schema's name.  (optional)
 
 =cut
 
-    my $self = shift;
-    $self->{'name'} = shift if @_;
-    return $self->{'name'} || '';
-}
-
-sub translator {
+has name => ( is => 'rw', default => quote_sub(q{ '' }) );
 
 =pod
 
@@ -769,16 +738,7 @@ Get the SQL::Translator instance that instantiated the parser.
 
 =cut
 
-    my $self = shift;
-    $self->{'translator'} = shift if @_;
-    return $self->{'translator'};
-}
-
-sub DESTROY {
-    my $self = shift;
-    undef $_ for values %{ $self->{'tables'} };
-    undef $_ for values %{ $self->{'views'} };
-}
+has translator => ( is => 'rw', weak_ref => 1 );
 
 1;
 
